#include "Cluster.h"
#include "Electron.h"
#include "Hit.h"
#include "MPGD.h"

#include <iostream>
#include <cstdlib>

ClassImp(GEN::Cluster)

namespace GEN {
//--
// C-tor for a cluster contributing to a hit
//   This C-tor tests if each electron in the src cluster contributes to the hit or not
//   and if it does it creates a copy cluster with only accepted electrons.
//--
Cluster::Cluster(Cluster *cp, // src cluster
                 Hit     *hp) // hit to which the cluster is to be added if within acceptance
{
  SetOwner(false); // cluster created with this C-tor is not allowed to own electrons
  if (!cp) return; // cluster must be given
  if (!hp) {       // hit should be specified
    std::cerr << "!!! Error in DetCluster::DetCluster !!!!!!!" << std::endl
              << " A cluster given but no hit specified!"      << std::endl
              << " --> abort! "                                << std::endl;
    exit(2);
  }

  ///FIXME///////////////////////////////////////////////////////
  static const int kNsig = 10; // allow +/-5-sigma fuzzy row edge
  ///FIXME///////////////////////////////////////////////////////
  double yacceptance = hp->GetMPGDPtr()->GetPadRowPitch()
                     + hp->GetMPGDPtr()->GetSigmaPRF()*kNsig;
  //--
  // Loop over electrons and add them if within acceptance
  //--
#ifdef DEBUG 
  std::cerr << "-------------------------------" << std::endl;
#endif
  TIter next(cp);
  while (auto ep = static_cast<Electron *>(next())) {
    if (fabs(ep->GetPosition().Y()) > yacceptance/2) continue; // it didn't reach this pad row
    TObjArray::Add(ep);
#ifdef DEBUG
    std::cerr << " e.xv = (" << ep->GetPosition().X() << ", "
	                     << ep->GetPosition().Y() << ", "
	                     << ep->GetPosition().Z() << ") "
              << std::endl;
#endif
  }
  if (!GetEntries()) return; // no electrons accepted
  //--
  // Copy relevant info from the source cluster
  //--
  fT      = cp->fT;     // creation time
  fX      = cp->fX;     // creation point
  fTrPtr  = cp->fTrPtr; // pointer to parent track
  fE      = cp->fE;     // energy deposit upon creation
  fClPtr  = cp;         // pointer to src cluster
  fHitPtr = hp;         // pointer to parent hit
}

//--
// Add method used to add electrons to a primary cluster beloinging to a Track.
//  Caution: this "Add" should not be used for a cluster beloinging to a Hit.
//--
void Cluster::Add(Electron *eptr) 
{
  TObjArray::Add(eptr);
  eptr->SetClPtr(this);
  fE += eptr->GetE();  
}
}
