#ifndef GEN_ELECTRON_H
#define GEN_ELECTRON_H
#include <TObject.h>
#include <TVector3.h>

namespace GEN {
class Cluster;
class MPGD;
class Hit;
class Electron : public TObject {
friend MPGD;
friend Hit;
friend Cluster;
public:
  Electron(double   t0 = 0.,
           TVector3 x0 = TVector3(0.,0.,0.),
           TVector3 d0 = TVector3(0.,0.,0.),
           double   e  = 0.,
           Cluster *clptr = nullptr)
          : fT0(t0), fX0(x0), fD0(d0), fE(e), fClPtr(clptr), fT(t0), fX(x0), fAmp(1.), fDone(false) {}  

  inline double    GetT0       () const { return fT0;    }
  inline TVector3  GetX0       () const { return fX0;    }
  inline TVector3  GetD0       () const { return fD0;    }
  inline double    GetE        () const { return fE;     }
  inline Cluster  *GetClPtr    () const { return fClPtr; }
  inline double    GetTime     () const { return fT;     }
  inline TVector3  GetPosition () const { return fX;     }
  inline double    GetAmpFactor() const { return fAmp;   }

  inline bool IsDone  () const { return fDone; } // true if drifed already

private:
  inline void SetClPtr(Cluster *clptr) { fClPtr = clptr; }

private:
         double    fT0;      // creation time [ns]
         TVector3  fX0;      // initial position of electron
         TVector3  fD0;      // initial direction of electron
         double    fE;       // energy of this electron
         Cluster  *fClPtr;   // pointer to parent cluster 
         double    fT;       // current time [ns]
         TVector3  fX;       // current electron position
	 double    fAmp;     // amplification factor (set by Hit object)
         bool      fDone;    // flag to tell if drift is done (set by MPGD::Drift)

  ClassDef(Electron, 1) // electron class
};
}
#endif
