#ifndef GEN_HIT_H
#define GEN_HIT_H
#include <TObjArray.h>

namespace GEN {
class Electron;
class Cluster;
class MPGD;
//------------------------------------------------------------------------------------
// ---------
// class Hit
// ---------
// (Description)
//   A class to describe a hit on a pad row with currently zero pad pitch.
//   It is an array of clusters consisting of seed electrons accepted by the pad row.
//------------------------------------------------------------------------------------
class Hit : public TObjArray {
public:
  Hit(MPGD *mpgdptr=nullptr) : fCsum(0), fNsum(0), fXcog(0.), fTcog(0.),
                               fDone(kFALSE), fMPGDPtr(mpgdptr) { SetOwner(); }

  double GetCharge() { if (!fDone) Update(); return fCsum; }
  double GetNsum  () { if (!fDone) Update(); return fNsum; }
  double GetXcog  () { if (!fDone) Update(); return fXcog; }
  double GetTcog  () { if (!fDone) Update(); return fTcog; }

  const MPGD *GetMPGDPtr() const;

  using TObjArray::Add;
  void Add(Cluster *cp);

private:
  void   Update(); // calculate C.O.G. etc.

private:
  double fCsum;      // total number of avalanche electrons 
  int    fNsum;      // total number of electrons
  double fXcog;      // charge center x [cm]
  double fTcog;      // average arrival time [ns]
  bool   fDone;      // false if update needed
  MPGD  *fMPGDPtr;   // pointer to parent MPGD

  ClassDef(Hit, 1) // hit class
};
}
#endif
