#include "MPGD.h"
#include "Track.h"
#include "Cluster.h"
#include "Electron.h"
#include "Polya.h"

#include <TRandom.h>
#include <TF2.h>

ClassImp(GEN::MPGD)
namespace GEN {
#if 0
TPC::Polya *MPGD::fgPolyaPtr = nullptr;
#else
GEN::Polya *MPGD::fgPolyaPtr = nullptr;
#endif
TF2        *MPGD::fgPrfPtr   = nullptr;

MPGD::MPGD(double dlenmax,     // Maximum Drift Length [cm]
           double padpitch,    // Pad Pitch [cm]
           double padrowpitch, // Pad Row Pitch [cm]
           double vdrift,      // Drift Velocity [cm/ns]
           double cdt,         // Transverse Diffusion Constant [cm/sqrt(cm)]
           double cdl,         // Longitudinal Diffusion Constant [cm/sqrt(cm)]
           double avgasgain,   // Gas Gain Average
           double theta,       // Gas Gain Fluctuation Parameter (Polya)
           double sigmaprf)    // PRF width [cm]
      : fDlenMax(dlenmax),
	fPadPitch(padpitch),
	fPadRowPitch(padrowpitch), 
        fDriftVelocity(vdrift),
	fCdT(cdt),
	fCdL(cdl), 
	fGasGainAverage(avgasgain),
	fPolyaTheta(theta),
	fSigmaPRF(sigmaprf)
{
#if 0
  if (!fgPolyaPtr) fgPolyaPtr = new TPC::Polya(theta);
#else
  if (!fgPolyaPtr) fgPolyaPtr = new GEN::Polya(theta);
#endif
  else             fgPolyaPtr->SetParameter(0,theta);

  if (!fgPrfPtr) {
    fgPrfPtr = new TF2("prf2d","(1./(2*TMath::Pi()*[0]*[0]))*exp(-(x*x+y*y)/(2*[0]*[0]))", -2.,2.,-2.,2.);
  }
  fgPrfPtr->SetParameter(0,sigmaprf);
}

void MPGD::Drift(Electron &e)
{
  if (e.fDone) return;
  
  double z   = e.GetX0().Z(); 
#if 1 //FIXME
         z   = z > 0. ? z : 0.;
#endif
  double sgT = fCdT * TMath::Sqrt(z);
  double sgL = fCdL * TMath::Sqrt(z);
  double x   = e.GetX0().X() + sgT * gRandom->Gaus(); 
  double y   = e.GetX0().Y() + sgT * gRandom->Gaus(); 

  double t0  = e.GetT0(); 
  double tda = (z + sgL * gRandom->Gaus()) / fDriftVelocity;

  z = 0.;
  
  e.fX = TVector3(x,y,z);
  e.fT = t0 + tda;

  e.fDone = true;
  return;
}

void MPGD::Drift(Track &t)
{
  if (t.fDone) return;
  TIter nextCl(&t);
  while (auto clp = static_cast<Cluster *>(nextCl())) {
    TIter nextEl(clp);
    while (auto elp = static_cast<Electron *>(nextEl())) {
      Drift(*elp);
    } 
  }
  t.fDone = true;
  return;
}

double MPGD::GasAmplify()
{
  return fGasGainAverage*fgPolyaPtr->GetRandom();
}

double MPGD::Prf(const double *x, const double *p)
{
  return fgPrfPtr->EvalPar(x,p);
}

double MPGD::Prf(int a, double x, double y)
{
  double xmin = fPadPitch*(a-0.5) - x;
  double xmax = fPadPitch*(a+0.5) - x;

  double ymin = -fPadRowPitch/2 - y;
  double ymax = +fPadRowPitch/2 - y;

  fgPrfPtr->SetParameter(0,fSigmaPRF);
  return fgPrfPtr->Integral(xmin,xmax,ymin,ymax);
}

double MPGD::Prf(double y)
{
  double xmin = -5.*fSigmaPRF;
  double xmax = -xmin;

  double ymin = -fPadRowPitch/2 - y;
  double ymax = +fPadRowPitch/2 - y;

  fgPrfPtr->SetParameter(0,fSigmaPRF);
  return fgPrfPtr->Integral(xmin,xmax,ymin,ymax);
}
}
