#ifndef GEN_TRACK_H
#define GEN_TRACK_H
#include <TObjArray.h>
#include <TVector3.h>

namespace GEN {
class Electron;
class Cluster;
class MPGD;

class Track : public TObjArray {
friend MPGD;
public:
  Track(const double      t0   = 0.,
        const TVector3    x0   = TVector3(0.,0.,0.),
        const TVector3    p0   = TVector3(0., 5., 0.),
        const std::string name = "electron",
        const double      tlen = 2.) //[cm]
        : fPName(name), fT0(t0), fX0(x0), fP0(p0), fE(0.), fTlen(tlen) ,fDone(false) { SetOwner(); }  

  inline std::string GetParticleName () const { return fPName; }
  inline double      GetT0           () const { return fT0;    }
  inline TVector3    GetX0           () const { return fX0;    }
  inline TVector3    GetP0           () const { return fP0;    }
  inline double      GetDepositEnergy() const { return fE;     }
  inline double      GetTrackLength  () const { return fTlen;  }

  inline bool IsDone  () const { return fDone; } // true if drifted already

  using TObjArray::Add;
  void Add(Cluster *);

private:
         std::string fPName;   // Particle name 
         double      fT0;      // initial time [ns]
         TVector3    fX0;      // initial position of electron
         TVector3    fP0;      // initial momentum of electron
         double      fE;       // total deposit energy
         double      fTlen;    // Track length [cm] (added on 2021/08/31: v2)
         bool        fDone;    // flag to tell if drift is done

  ClassDef(Track, 2) // track class
};
}
#endif
