#!/bin/sh
# =========
#  neff.sh
# =========
#
# (Function)
#   Create n-tuples from generated track data stored in $datadir for neff calculations.
#   By default datadir is set to Data.
# (Usage)
#   sh neff.sh <maxevt> <datadir>
# (Upadate Record)
#   2021/03/06   K.Fujii         Original version.
#
#--
#  Check arguments.
#--
if [ $# -ge 3 ]; then
      echo "Usage:"
      echo "  sh neff.sh <maxevt> <datadir>"
      echo " where <maxevt> is the maximum number of events to generate"
      echo " per run with its default value being 10000."
      exit 0
#--
# Set maximum number of events to reconstruct per run
#--
elif [ $# -ge 1 ]; then
      echo "maxevt is set to $1"
      maxevt=$1
      if [ $# -eq 2 ]; then
              echo "datadir is set to $2"
              datadir=$2
      fi
else
      maxevt=10000
      datadir=Data
fi

#--
# Set maximum number of events to reconstruct per run
#--
echo "Going to process $maxevt events per run"
for i in $datadir/edep.*.root; do
      logname=`basename $i .root | sed -e 's;^edep;neff;'`.log
      echo " ... processing $i ..."
      #eval "root -l -q -b neff.C\(\\\"$i\\\",$maxevt\) >& $logname &"
      bsub -J "neff:$i" -o $logname "source ./setup.sh && root -l -q -b neff.C\(\\\"$i\\\",$maxevt\)"
done
#wait
#--
# That's it, folks!
#--
exit 0
